use argon2::{
    password_hash::{rand_core::OsRng, PasswordHasher, SaltString},
    Argon2, PasswordHash, PasswordVerifier,
};
use axum::http::{header, HeaderMap};
use chrono::Utc;
pub use error::AuthError;
use rand::distr::Alphanumeric;
use rand::prelude::*;
use rand_chacha::ChaCha20Rng;
use sqlx::PgPool;
use tokio::task;

use crate::{database::model::Session, model::User};

mod error;

pub async fn get_user_from_header(pool: &PgPool, headers: &HeaderMap) -> Result<User, AuthError> {
    let token = get_token_from_headers(&headers)?;

    let session = match Session::from_token(pool, &token).await {
        Ok(s) => s,
        Err(_) => return Err(AuthError::InvalidToken),
    };

    if session.expires_at < Utc::now() {
        return Err(AuthError::InvalidToken);
    }

    let db_user = match crate::database::model::User::get(pool, session.user_id).await {
        Ok(u) => u,
        Err(_) => return Err(AuthError::InvalidToken),
    };

    Ok(db_user.into())
}

pub fn get_token_from_headers(headers: &HeaderMap) -> Result<String, AuthError> {
    let bearer = headers.get(header::AUTHORIZATION);
    let bearer = bearer
        .ok_or(AuthError::InvalidToken)?
        .to_str()
        .map_err(|_| AuthError::InvalidToken)?;

    match bearer.strip_prefix("Bearer ") {
        Some(token) => Ok(token.to_string()),
        None => Err(AuthError::InvalidToken),
    }
}

pub async fn generate_password_hash(password: String) -> Result<String, AuthError> {
    let password_hash: Result<String, argon2::password_hash::Error> =
        task::spawn_blocking(move || {
            let salt = SaltString::generate(&mut OsRng);

            let argon2 = Argon2::default();

            let password_hash = argon2
                .hash_password(password.as_bytes(), &salt)?
                .to_string();

            Ok(password_hash)
        })
        .await
        .unwrap();

    password_hash.map_err(|e| e.into())
}

pub async fn verify_password_hash(password: &str, hash: &str) -> Result<(), AuthError> {
    let parsed_hash = PasswordHash::new(hash)?;
    Argon2::default()
        .verify_password(password.as_bytes(), &parsed_hash)
        .map_err(|_| AuthError::InvalidPassword)?;

    Ok(())
}

pub fn generate_session_token() -> String {
    ChaCha20Rng::from_os_rng()
        .sample_iter(&Alphanumeric)
        .take(60)
        .map(char::from)
        .collect::<String>()
}
