use chrono::{DateTime, Utc};
use sqlx::{PgPool, Postgres};

#[derive(Debug)]
pub struct Session {
    pub session_id: uuid::Uuid,
    pub user_id: uuid::Uuid,
    pub token: String,
    pub expires_at: DateTime<Utc>,
    pub created_at: DateTime<Utc>,
}

impl Session {
    pub async fn insert(
        &self,
        transaction: &mut sqlx::Transaction<'_, Postgres>,
    ) -> Result<(), sqlx::Error> {
        sqlx::query!(
            "
                INSERT INTO sessions (
                    session_id, user_id, token, expires_at, created_at
                )
                VALUES (
                    $1, $2, $3, $4, $5
                )
            ",
            &self.session_id,
            &self.user_id,
            &self.token,
            &self.expires_at,
            &self.created_at
        )
        .execute(&mut **transaction)
        .await?;

        Ok(())
    }

    pub async fn from_token(pool: &PgPool, token: &str) -> Result<Self, sqlx::Error> {
        let session = sqlx::query_as!(Self, "SELECT * FROM sessions WHERE token = $1;", token)
            .fetch_one(pool)
            .await?;

        Ok(session)
    }

    pub async fn remove_many(
        session_ids: &[uuid::Uuid],
        transaction: &mut sqlx::Transaction<'_, Postgres>,
    ) -> Result<(), sqlx::Error> {
        let deleted_count = sqlx::query_scalar!(
            "
                WITH deleted AS (
                    DELETE FROM sessions
                    WHERE session_id = ANY($1)
                    RETURNING 1
                )
                SELECT COUNT(*) FROM deleted
            ",
            session_ids
        )
        .fetch_one(&mut **transaction)
        .await?;

        if !deleted_count.is_some_and(|c| c >= 1) {
            return Err(sqlx::Error::RowNotFound);
        }

        Ok(())
    }

    pub async fn remove_many_from_token(
        transaction: &mut sqlx::Transaction<'_, Postgres>,
        session_tokens: &[String],
    ) -> Result<(), sqlx::Error> {
        let deleted_count = sqlx::query_scalar!(
            "
                WITH deleted AS (
                    DELETE FROM sessions
                    WHERE token = ANY($1)
                    RETURNING 1
                )
                SELECT COUNT(*) FROM deleted
            ",
            session_tokens
        )
        .fetch_one(&mut **transaction)
        .await?;

        if !deleted_count.is_some_and(|c| c >= 1) {
            return Err(sqlx::Error::RowNotFound);
        }

        Ok(())
    }
}
